@@ -7,7 +7,6 @@ import json
 import attr
 
 from requests import HTTPError
-from six import raise_from
 from more_executors import Executors
 from cloudimg.aws import AWSPublishingMetadata
 from pushsource import Source, AmiPushItem
@@ -219,6 +218,12 @@ class AmiPush(AmiTask, RHSMClientService, AWSPublishService, CollectorService):
         else:
             accounts = list(_accounts["default"].values())
 
+        snapshot_account_ids = (
+            self.args.snapshot_account_ids[region]
+            if region in self.args.snapshot_account_ids
+            else self.args.snapshot_account_ids["default"]
+        )
+
         publishing_meta_kwargs = {
             "image_path": file_path,
             "image_name": name,
@@ -231,7 +236,7 @@ class AmiPush(AmiTask, RHSMClientService, AWSPublishService, CollectorService):
             "volume_type": push_item.volume,
             "billing_products": push_item.billing_codes.codes,
             "accounts": accounts,
-            "snapshot_account_ids": self.args.snapshot_account_ids,
+            "snapshot_account_ids": snapshot_account_ids,
             "sriov_net_support": push_item.sriov_net_support,
             "ena_support": push_item.ena_support or False,
         }
@@ -242,23 +247,28 @@ class AmiPush(AmiTask, RHSMClientService, AWSPublishService, CollectorService):
         try:
             image = aws.publish(publish_meta)
         except Exception as exc:  # pylint:disable=broad-except
-            raise_from(AWSPublishError(exc), exc)
+            raise AWSPublishError(exc) from exc
 
         if ship:
             self.update_rhsm_metadata(image, push_item)
 
-            if image_type == "hourly" and self.args.allow_public_images:
-                LOG.info("Releasing hourly image %s publicly", image.id)
+            public_image = push_item.public_image
+            if public_image is None and image_type == "hourly":
+                # Backwards compatibility for push items without public_image flag.
                 # The "all" group grants access to the general public. This
                 # should only be done for hourly images since they are the only
                 # type to charge an additional Red Hat fee.
                 # http://docs.aws.amazon.com/AWSEC2/latest/UserGuide/sharingamis-intro.html
+                public_image = True
+
+            if public_image and self.args.allow_public_images:
+                LOG.info("Releasing image %s publicly", image.id)
                 publish_meta.groups = ["all"]
                 # A repeat call to publish will only update the groups
                 try:
                     aws.publish(publish_meta)
                 except Exception as exc:  # pylint:disable=broad-except
-                    raise_from(AWSPublishError(exc), exc)
+                    raise AWSPublishError(exc) from exc
 
         LOG.info("Successfully uploaded %s [%s] [%s]", name, region, image.id)
 
@@ -452,12 +462,11 @@ class AmiPush(AmiTask, RHSMClientService, AWSPublishService, CollectorService):
 
         group.add_argument(
             "--snapshot-account-ids",
-            help="Comma separated list of account ids to give snapshot "
-            "creation permissions to if a new snapshot is created as "
-            "part of the image push.",
-            action=SplitAndExtend,
-            split_on=",",
-            remove_duplicates=True,
+            help="JSON string mapping region to a list of account ids to give "
+            "snapshot creation permissions to if a new snapshot is created "
+            "as part of the image push.",
+            type=json.loads,
+            default={},
         )
 
     def run(self):
